@@ -19,16 +19,37 @@
 #ifndef __VKD3D_D3D12_TEST_UTILS_H
 #define __VKD3D_D3D12_TEST_UTILS_H
 
+#include "utils.h"
+
 #define wait_queue_idle(a, b) wait_queue_idle_(__LINE__, a, b)
 static void wait_queue_idle_(unsigned int line, ID3D12Device *device, ID3D12CommandQueue *queue);
 static ID3D12Device *create_device(void);
 
-static void set_rect(RECT *rect, int left, int top, int right, int bottom)
+static inline HRESULT wait_for_fence_no_event(ID3D12Fence *fence, uint64_t value)
 {
-    rect->left = left;
-    rect->right = right;
-    rect->top = top;
-    rect->bottom = bottom;
+    if (ID3D12Fence_GetCompletedValue(fence) >= value)
+        return S_OK;
+
+    /* This is defined to block on the value with infinite timeout. */
+    return ID3D12Fence_SetEventOnCompletion(fence, value, NULL);
+}
+
+#define wait_queue_idle_no_event(a, b) wait_queue_idle_no_event_(__LINE__, a, b)
+static inline void wait_queue_idle_no_event_(unsigned int line, ID3D12Device *device, ID3D12CommandQueue *queue)
+{
+    ID3D12Fence *fence;
+    HRESULT hr;
+
+    hr = ID3D12Device_CreateFence(device, 0, D3D12_FENCE_FLAG_NONE,
+        &IID_ID3D12Fence, (void **)&fence);
+    assert_that_(line)(hr == S_OK, "Failed to create fence, hr %#x.\n", hr);
+
+    hr = ID3D12CommandQueue_Signal(queue, fence, 1);
+    assert_that_(line)(hr == S_OK, "Failed to signal fence, hr %#x.\n", hr);
+    hr = wait_for_fence_no_event(fence, 1);
+    assert_that_(line)(hr == S_OK, "Failed to wait for fence, hr %#x.\n", hr);
+
+    ID3D12Fence_Release(fence);
 }
 
 static inline void set_box(D3D12_BOX *box, unsigned int left, unsigned int top, unsigned int front,
@@ -307,6 +328,7 @@ static unsigned int format_size(DXGI_FORMAT format)
         case DXGI_FORMAT_R8G8B8A8_UNORM_SRGB:
         case DXGI_FORMAT_R8G8B8A8_UINT:
         case DXGI_FORMAT_B8G8R8A8_UNORM:
+        case DXGI_FORMAT_D24_UNORM_S8_UINT:
             return 4;
         case DXGI_FORMAT_R16_FLOAT:
         case DXGI_FORMAT_R16_UNORM:
@@ -389,18 +411,14 @@ static inline unsigned int format_block_height(DXGI_FORMAT format)
     }
 }
 
-struct resource_readback
+struct d3d12_resource_readback
 {
-    uint64_t width;
-    unsigned int height;
-    unsigned int depth;
+    struct resource_readback rb;
     ID3D12Resource *resource;
-    uint64_t row_pitch;
-    void *data;
 };
 
 static void get_texture_readback_with_command_list(ID3D12Resource *texture, unsigned int sub_resource,
-        struct resource_readback *rb, ID3D12CommandQueue *queue, ID3D12GraphicsCommandList *command_list)
+        struct d3d12_resource_readback *rb, ID3D12CommandQueue *queue, ID3D12GraphicsCommandList *command_list)
 {
     D3D12_TEXTURE_COPY_LOCATION dst_location, src_location;
     D3D12_HEAP_PROPERTIES heap_properties;
@@ -420,14 +438,14 @@ static void get_texture_readback_with_command_list(ID3D12Resource *texture, unsi
             "Resource %p is not texture.\n", texture);
 
     miplevel = sub_resource % resource_desc.MipLevels;
-    rb->width = max(1, resource_desc.Width >> miplevel);
-    rb->width = align(rb->width, format_block_width(resource_desc.Format));
-    rb->height = max(1, resource_desc.Height >> miplevel);
-    rb->height = align(rb->height, format_block_height(resource_desc.Format));
-    rb->depth = resource_desc.Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE3D
+    rb->rb.width = max(1, resource_desc.Width >> miplevel);
+    rb->rb.width = align(rb->rb.width, format_block_width(resource_desc.Format));
+    rb->rb.height = max(1, resource_desc.Height >> miplevel);
+    rb->rb.height = align(rb->rb.height, format_block_height(resource_desc.Format));
+    rb->rb.depth = resource_desc.Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE3D
             ? max(1, resource_desc.DepthOrArraySize >> miplevel) : 1;
-    rb->row_pitch = align(rb->width * format_size(resource_desc.Format), D3D12_TEXTURE_DATA_PITCH_ALIGNMENT);
-    rb->data = NULL;
+    rb->rb.row_pitch = align(rb->rb.width * format_size(resource_desc.Format), D3D12_TEXTURE_DATA_PITCH_ALIGNMENT);
+    rb->rb.data = NULL;
 
     if (resource_desc.SampleDesc.Count > 1)
     {
@@ -455,17 +473,17 @@ static void get_texture_readback_with_command_list(ID3D12Resource *texture, unsi
         src_resource = texture;
     }
 
-    buffer_size = rb->row_pitch * rb->height * rb->depth;
+    buffer_size = rb->rb.row_pitch * rb->rb.height * rb->rb.depth;
     rb->resource = create_readback_buffer(device, buffer_size);
 
     dst_location.pResource = rb->resource;
     dst_location.Type = D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT;
     dst_location.PlacedFootprint.Offset = 0;
     dst_location.PlacedFootprint.Footprint.Format = resource_desc.Format;
-    dst_location.PlacedFootprint.Footprint.Width = rb->width;
-    dst_location.PlacedFootprint.Footprint.Height = rb->height;
-    dst_location.PlacedFootprint.Footprint.Depth = rb->depth;
-    dst_location.PlacedFootprint.Footprint.RowPitch = rb->row_pitch;
+    dst_location.PlacedFootprint.Footprint.Width = rb->rb.width;
+    dst_location.PlacedFootprint.Footprint.Height = rb->rb.height;
+    dst_location.PlacedFootprint.Footprint.Depth = rb->rb.depth;
+    dst_location.PlacedFootprint.Footprint.RowPitch = rb->rb.row_pitch;
 
     src_location.pResource = src_resource;
     src_location.Type = D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX;
@@ -484,24 +502,17 @@ static void get_texture_readback_with_command_list(ID3D12Resource *texture, unsi
 
     read_range.Begin = 0;
     read_range.End = buffer_size;
-    hr = ID3D12Resource_Map(rb->resource, 0, &read_range, &rb->data);
+    hr = ID3D12Resource_Map(rb->resource, 0, &read_range, &rb->rb.data);
     assert_that(hr == S_OK, "Failed to map readback buffer, hr %#x.\n", hr);
 }
 
-static void *get_readback_data(struct resource_readback *rb,
-        unsigned int x, unsigned int y, unsigned int z, size_t element_size)
-{
-    unsigned int slice_pitch = rb->row_pitch * rb->height;
-    return &((BYTE *)rb->data)[slice_pitch * z + rb->row_pitch * y + x * element_size];
-}
-
 static unsigned int get_readback_uint(struct resource_readback *rb,
         unsigned int x, unsigned int y, unsigned int z)
 {
     return *(unsigned int *)get_readback_data(rb, x, y, z, sizeof(unsigned int));
 }
 
-static void release_resource_readback(struct resource_readback *rb)
+static void release_resource_readback(struct d3d12_resource_readback *rb)
 {
     D3D12_RANGE range = {0, 0};
     ID3D12Resource_Unmap(rb->resource, 0, &range);
@@ -547,10 +558,22 @@ static inline void check_sub_resource_uint_(unsigned int line, ID3D12Resource *t
         unsigned int sub_resource_idx, ID3D12CommandQueue *queue, ID3D12GraphicsCommandList *command_list,
         unsigned int expected, unsigned int max_diff)
 {
-    struct resource_readback rb;
+    struct d3d12_resource_readback rb;
 
     get_texture_readback_with_command_list(texture, sub_resource_idx, &rb, queue, command_list);
-    check_readback_data_uint_(line, &rb, NULL, expected, max_diff);
+    check_readback_data_uint_(line, &rb.rb, NULL, expected, max_diff);
+    release_resource_readback(&rb);
+}
+
+#define check_sub_resource_vec4(a, b, c, d, e, f) check_sub_resource_vec4_(__LINE__, a, b, c, d, e, f)
+static inline void check_sub_resource_vec4_(unsigned int line, ID3D12Resource *texture,
+        unsigned int sub_resource_idx, ID3D12CommandQueue *queue, ID3D12GraphicsCommandList *command_list,
+        const struct vec4 *expected, unsigned int max_diff)
+{
+    struct d3d12_resource_readback rb;
+
+    get_texture_readback_with_command_list(texture, sub_resource_idx, &rb, queue, command_list);
+    check_readback_data_vec4_(line, &rb.rb, NULL, expected, max_diff);
     release_resource_readback(&rb);
 }
 
@@ -595,7 +618,7 @@ static ID3D12Resource *create_default_texture_(unsigned int line, ID3D12Device *
 
 #define create_default_texture(a, b, c, d, e, f) create_default_texture2d_(__LINE__, a, b, c, 1, 1, d, e, f)
 #define create_default_texture2d(a, b, c, d, e, f, g, h) create_default_texture2d_(__LINE__, a, b, c, d, e, f, g, h)
-static ID3D12Resource *create_default_texture2d_(unsigned int line, ID3D12Device *device,
+static inline ID3D12Resource *create_default_texture2d_(unsigned int line, ID3D12Device *device,
         unsigned int width, unsigned int height, unsigned int array_size, unsigned int miplevel_count,
         DXGI_FORMAT format, D3D12_RESOURCE_FLAGS flags, D3D12_RESOURCE_STATES initial_state)
 {
@@ -612,6 +635,95 @@ static inline ID3D12Resource *create_default_texture3d_(unsigned int line, ID3D1
             width, height, depth, miplevel_count, format, flags, initial_state);
 }
 
+static void copy_sub_resource_data(const D3D12_MEMCPY_DEST *dst, const D3D12_SUBRESOURCE_DATA *src,
+        unsigned int row_count, unsigned int slice_count, size_t row_size)
+{
+    const BYTE *src_slice_ptr;
+    BYTE *dst_slice_ptr;
+    unsigned int z, y;
+
+    for (z = 0; z < slice_count; ++z)
+    {
+        dst_slice_ptr = (BYTE *)dst->pData + z * dst->SlicePitch;
+        src_slice_ptr = (const BYTE*)src->pData + z * src->SlicePitch;
+        for (y = 0; y < row_count; ++y)
+            memcpy(dst_slice_ptr + y * dst->RowPitch, src_slice_ptr + y * src->RowPitch, row_size);
+    }
+}
+
+#define upload_texture_data(a, b, c, d, e) upload_texture_data_(__LINE__, a, b, c, d, e)
+static inline void upload_texture_data_(unsigned int line, ID3D12Resource *texture,
+        const D3D12_SUBRESOURCE_DATA *data, unsigned int sub_resource_count,
+        ID3D12CommandQueue *queue, ID3D12GraphicsCommandList *command_list)
+{
+    D3D12_TEXTURE_COPY_LOCATION dst_location, src_location;
+    D3D12_PLACED_SUBRESOURCE_FOOTPRINT *layouts;
+    uint64_t *row_sizes, required_size;
+    D3D12_RESOURCE_DESC resource_desc;
+    ID3D12Resource *upload_buffer;
+    D3D12_MEMCPY_DEST dst_data;
+    ID3D12Device *device;
+    UINT *row_counts;
+    unsigned int i;
+    HRESULT hr;
+    void *ptr;
+
+    layouts = calloc(sub_resource_count, sizeof(*layouts));
+    ok(layouts, "Failed to allocate memory.\n");
+    row_counts = calloc(sub_resource_count, sizeof(*row_counts));
+    ok(row_counts, "Failed to allocate memory.\n");
+    row_sizes = calloc(sub_resource_count, sizeof(*row_sizes));
+    ok(row_sizes, "Failed to allocate memory.\n");
+
+    resource_desc = ID3D12Resource_GetDesc(texture);
+    hr = ID3D12Resource_GetDevice(texture, &IID_ID3D12Device, (void **)&device);
+    ok_(line)(SUCCEEDED(hr), "Failed to get device, hr %#x.\n", hr);
+
+    ID3D12Device_GetCopyableFootprints(device, &resource_desc, 0, sub_resource_count,
+            0, layouts, row_counts, row_sizes, &required_size);
+
+    upload_buffer = create_upload_buffer_(line, device, required_size, NULL);
+
+    hr = ID3D12Resource_Map(upload_buffer, 0, NULL, (void **)&ptr);
+    ok_(line)(SUCCEEDED(hr), "Failed to map upload buffer, hr %#x.\n", hr);
+    for (i = 0; i < sub_resource_count; ++i)
+    {
+        dst_data.pData = (BYTE *)ptr + layouts[i].Offset;
+        dst_data.RowPitch = layouts[i].Footprint.RowPitch;
+        dst_data.SlicePitch = layouts[i].Footprint.RowPitch * row_counts[i];
+        copy_sub_resource_data(&dst_data, &data[i],
+                row_counts[i], layouts[i].Footprint.Depth, row_sizes[i]);
+    }
+    ID3D12Resource_Unmap(upload_buffer, 0, NULL);
+
+    for (i = 0; i < sub_resource_count; ++i)
+    {
+        dst_location.pResource = texture;
+        dst_location.Type = D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX;
+        dst_location.SubresourceIndex = i;
+
+        src_location.pResource = upload_buffer;
+        src_location.Type = D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT;
+        src_location.PlacedFootprint = layouts[i];
+
+        ID3D12GraphicsCommandList_CopyTextureRegion(command_list,
+                &dst_location, 0, 0, 0, &src_location, NULL);
+    }
+
+    hr = ID3D12GraphicsCommandList_Close(command_list);
+    ok_(line)(SUCCEEDED(hr), "Failed to close command list, hr %#x.\n", hr);
+
+    exec_command_list(queue, command_list);
+    wait_queue_idle(device, queue);
+
+    ID3D12Resource_Release(upload_buffer);
+    ID3D12Device_Release(device);
+
+    free(layouts);
+    free(row_counts);
+    free(row_sizes);
+}
+
 static HRESULT create_root_signature(ID3D12Device *device, const D3D12_ROOT_SIGNATURE_DESC *desc,
         ID3D12RootSignature **root_signature)
 {
@@ -646,6 +758,33 @@ static ID3D12RootSignature *create_empty_root_signature_(unsigned int line,
     return root_signature;
 }
 
+#define create_32bit_constants_root_signature(a, b, c, e) \
+        create_32bit_constants_root_signature_(__LINE__, a, b, c, e, 0)
+static inline ID3D12RootSignature *create_32bit_constants_root_signature_(unsigned int line,
+        ID3D12Device *device, unsigned int reg_idx, unsigned int element_count,
+        D3D12_SHADER_VISIBILITY shader_visibility, D3D12_ROOT_SIGNATURE_FLAGS flags)
+{
+    D3D12_ROOT_SIGNATURE_DESC root_signature_desc;
+    ID3D12RootSignature *root_signature = NULL;
+    D3D12_ROOT_PARAMETER root_parameter;
+    HRESULT hr;
+
+    root_parameter.ParameterType = D3D12_ROOT_PARAMETER_TYPE_32BIT_CONSTANTS;
+    root_parameter.Constants.ShaderRegister = reg_idx;
+    root_parameter.Constants.RegisterSpace = 0;
+    root_parameter.Constants.Num32BitValues = element_count;
+    root_parameter.ShaderVisibility = shader_visibility;
+
+    memset(&root_signature_desc, 0, sizeof(root_signature_desc));
+    root_signature_desc.NumParameters = 1;
+    root_signature_desc.pParameters = &root_parameter;
+    root_signature_desc.Flags = flags;
+    hr = create_root_signature(device, &root_signature_desc, &root_signature);
+    ok_(line)(SUCCEEDED(hr), "Failed to create root signature, hr %#x.\n", hr);
+
+    return root_signature;
+}
+
 static void init_pipeline_state_desc(D3D12_GRAPHICS_PIPELINE_STATE_DESC *desc,
         ID3D12RootSignature *root_signature, DXGI_FORMAT rt_format,
         const D3D12_SHADER_BYTECODE *vs, const D3D12_SHADER_BYTECODE *ps,
@@ -724,7 +863,7 @@ static ID3D12PipelineState *create_pipeline_state_(unsigned int line, ID3D12Devi
         const D3D12_INPUT_LAYOUT_DESC *input_layout)
 {
     D3D12_GRAPHICS_PIPELINE_STATE_DESC pipeline_state_desc;
-    ID3D12PipelineState *pipeline_state;
+    ID3D12PipelineState *pipeline_state = NULL;
     HRESULT hr;
 
     init_pipeline_state_desc(&pipeline_state_desc, root_signature, rt_format, vs, ps, input_layout);
@@ -732,6 +871,26 @@ static ID3D12PipelineState *create_pipeline_state_(unsigned int line, ID3D12Devi
             &IID_ID3D12PipelineState, (void **)&pipeline_state);
     ok_(line)(SUCCEEDED(hr), "Failed to create graphics pipeline state, hr %#x.\n", hr);
 
+    return SUCCEEDED(hr) ? pipeline_state : NULL;
+}
+
+#define create_compute_pipeline_state(a, b, c) create_compute_pipeline_state_(__LINE__, a, b, c)
+static inline ID3D12PipelineState *create_compute_pipeline_state_(unsigned int line, ID3D12Device *device,
+        ID3D12RootSignature *root_signature, const D3D12_SHADER_BYTECODE cs)
+{
+    D3D12_COMPUTE_PIPELINE_STATE_DESC pipeline_state_desc;
+    ID3D12PipelineState *pipeline_state = NULL;
+    HRESULT hr;
+
+    memset(&pipeline_state_desc, 0, sizeof(pipeline_state_desc));
+    pipeline_state_desc.pRootSignature = root_signature;
+    pipeline_state_desc.CS = cs;
+    pipeline_state_desc.NodeMask = 0;
+    pipeline_state_desc.Flags = D3D12_PIPELINE_STATE_FLAG_NONE;
+    hr = ID3D12Device_CreateComputePipelineState(device, &pipeline_state_desc,
+            &IID_ID3D12PipelineState, (void **)&pipeline_state);
+    ok_(line)(SUCCEEDED(hr), "Failed to create compute pipeline state, hr %#x.\n", hr);
+
     return pipeline_state;
 }
 
@@ -755,6 +914,8 @@ struct test_context
     ID3D12CommandQueue *queue;
     ID3D12CommandAllocator *allocator;
     ID3D12GraphicsCommandList *list;
+    ID3D12PipelineState **pso;
+    size_t pso_count, pso_capacity;
 
     D3D12_RESOURCE_DESC render_target_desc;
     ID3D12Resource *render_target;
@@ -873,6 +1034,15 @@ static inline bool init_test_context_(unsigned int line, struct test_context *co
     return true;
 }
 
+static inline void destroy_pipeline_state_objects(struct test_context *context)
+{
+    size_t i;
+
+    for (i = 0; i < context->pso_count; ++i)
+        ID3D12PipelineState_Release(context->pso[i]);
+    context->pso_count = 0;
+}
+
 #define destroy_test_context(context) destroy_test_context_(__LINE__, context)
 static inline void destroy_test_context_(unsigned int line, struct test_context *context)
 {
@@ -891,6 +1061,8 @@ static inline void destroy_test_context_(unsigned int line, struct test_context
     ID3D12CommandAllocator_Release(context->allocator);
     ID3D12CommandQueue_Release(context->queue);
     ID3D12GraphicsCommandList_Release(context->list);
+    destroy_pipeline_state_objects(context);
+    free(context->pso);
 
     refcount = ID3D12Device_Release(context->device);
     ok_(line)(!refcount, "ID3D12Device has %u references left.\n", (unsigned int)refcount);
